@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2022 The Qt Company Ltd.
 ** Copyright (C) 2016 Intel Corporation.
 ** Contact: https://www.qt.io/licensing/
 **
@@ -83,15 +83,15 @@
 #  endif
 
 #elif defined(_MSC_VER)
-#  ifdef __clang__
-#    define Q_CC_CLANG ((__clang_major__ * 100) + __clang_minor__)
-#  endif
 #  define Q_CC_MSVC (_MSC_VER)
 #  define Q_CC_MSVC_NET
-#  define Q_OUTOFLINE_TEMPLATE inline
-#  if _MSC_VER < 1600
-#    define Q_NO_TEMPLATE_FRIENDS
+#  define Q_CC_MSVC_ONLY Q_CC_MSVC
+#  ifdef __clang__
+#    undef Q_CC_MSVC_ONLY
+#    define Q_CC_CLANG ((__clang_major__ * 100) + __clang_minor__)
+#    define Q_CC_CLANG_ONLY Q_CC_CLANG
 #  endif
+#  define Q_OUTOFLINE_TEMPLATE inline
 #  define Q_COMPILER_MANGLES_RETURN_TYPE
 #  define Q_FUNC_INFO __FUNCSIG__
 #  define Q_ALIGNOF(type) __alignof(type)
@@ -105,24 +105,18 @@
 #  endif
 #  define Q_DECL_EXPORT __declspec(dllexport)
 #  define Q_DECL_IMPORT __declspec(dllimport)
-#  if _MSC_VER >= 1800
-#    define QT_MAKE_UNCHECKED_ARRAY_ITERATOR(x) stdext::make_unchecked_array_iterator(x)
-#  endif
-#  if _MSC_VER >= 1500
-#    define QT_MAKE_CHECKED_ARRAY_ITERATOR(x, N) stdext::make_checked_array_iterator(x, size_t(N))
-#  endif
+#  define QT_MAKE_UNCHECKED_ARRAY_ITERATOR(x) stdext::make_unchecked_array_iterator(x) // Since _MSC_VER >= 1800
+#  define QT_MAKE_CHECKED_ARRAY_ITERATOR(x, N) stdext::make_checked_array_iterator(x, size_t(N)) // Since _MSC_VER >= 1500
 /* Intel C++ disguising as Visual C++: the `using' keyword avoids warnings */
 #  if defined(__INTEL_COMPILER)
+#    undef Q_CC_MSVC_ONLY
+#    ifdef Q_CC_CLANG_ONLY
+#      undef Q_CC_CLANG_ONLY
+#    endif
 #    define Q_DECL_VARIABLE_DEPRECATED
 #    define Q_CC_INTEL  __INTEL_COMPILER
 #  endif
 
-/* only defined for MSVC since that's the only compiler that actually optimizes for this */
-/* might get overridden further down when Q_COMPILER_NOEXCEPT is detected */
-#  ifdef __cplusplus
-#    define Q_DECL_NOTHROW  throw()
-#  endif
-
 #elif defined(__BORLANDC__) || defined(__TURBOC__)
 #  define Q_CC_BOR
 #  define Q_INLINE_TEMPLATE
@@ -196,6 +190,7 @@
 #    else
 #      define Q_CC_CLANG ((__clang_major__ * 100) + __clang_minor__)
 #    endif
+#    define Q_CC_CLANG_ONLY Q_CC_CLANG
 #    if __has_builtin(__builtin_assume)
 #      define Q_ASSUME_IMPL(expr)   __builtin_assume(expr)
 #    else
@@ -213,8 +208,12 @@
 #        define Q_DECL_NS_RETURNS_AUTORELEASED __attribute__((ns_returns_autoreleased))
 #      endif
 #    endif
+#    ifdef __EMSCRIPTEN__
+#      define Q_CC_EMSCRIPTEN
+#    endif
 #  else
 /* Plain GCC */
+#    define Q_CC_GNU_ONLY Q_CC_GNU
 #    if Q_CC_GNU >= 405
 #      define Q_ASSUME_IMPL(expr)  if (expr){} else __builtin_unreachable()
 #      define Q_UNREACHABLE_IMPL() __builtin_unreachable()
@@ -243,6 +242,7 @@
 #  define Q_REQUIRED_RESULT __attribute__ ((__warn_unused_result__))
 #  define Q_DECL_PURE_FUNCTION __attribute__((pure))
 #  define Q_DECL_CONST_FUNCTION __attribute__((const))
+#  define Q_DECL_COLD_FUNCTION __attribute__((cold))
 #  if !defined(QT_MOC_CPP)
 #    define Q_PACKED __attribute__ ((__packed__))
 #    ifndef __ARM_EABI__
@@ -374,7 +374,6 @@
 #        define Q_COMPILER_ATTRIBUTES
 #        define Q_COMPILER_AUTO_FUNCTION
 #        define Q_COMPILER_CLASS_ENUM
-#        define Q_COMPILER_CONSTEXPR
 #        define Q_COMPILER_DECLTYPE
 #        define Q_COMPILER_DEFAULT_MEMBERS
 #        define Q_COMPILER_DELETE_MEMBERS
@@ -514,6 +513,39 @@
 #  error "Qt has not been tested with this compiler - see http://www.qt-project.org/"
 #endif
 
+/*
+ * SG10's SD-6 feature detection and some useful extensions from Clang and GCC
+ * https://isocpp.org/std/standing-documents/sd-6-sg10-feature-test-recommendations
+ * http://clang.llvm.org/docs/LanguageExtensions.html#feature-checking-macros
+ * Not using wrapper macros, per http://eel.is/c++draft/cpp.cond#7.sentence-2
+ */
+#ifndef __has_builtin
+#  define __has_builtin(x)             0
+#endif
+#ifndef __has_feature
+#  define __has_feature(x)             0
+#endif
+#ifndef __has_attribute
+#  define __has_attribute(x)           0
+#endif
+#ifndef __has_cpp_attribute
+#  define __has_cpp_attribute(x)       0
+#endif
+#ifndef __has_include
+#  define __has_include(x)             0
+#endif
+#ifndef __has_include_next
+#  define __has_include_next(x)        0
+#endif
+
+// Kept around until all submodules have transitioned
+#define QT_HAS_BUILTIN(x)        __has_builtin(x)
+#define QT_HAS_FEATURE(x)        __has_feature(x)
+#define QT_HAS_ATTRIBUTE(x)      __has_attribute(x)
+#define QT_HAS_CPP_ATTRIBUTE(x)  __has_cpp_attribute(x)
+#define QT_HAS_INCLUDE(x)        __has_include(x)
+#define QT_HAS_INCLUDE_NEXT(x)   __has_include_next(x)
+
 /*
  * C++11 support
  *
@@ -563,7 +595,7 @@
  */
 
 #ifdef __cplusplus
-#  if __cplusplus < 201103L && !(defined(Q_CC_MSVC) && Q_CC_MSVC >= 1800)
+#  if __cplusplus < 201103L && !defined(Q_CC_MSVC)
 #    error Qt requires a C++11 compiler and yours does not seem to be that.
 #  endif
 #endif
@@ -635,21 +667,11 @@
 #      define Q_COMPILER_THREAD_LOCAL
 #      define Q_COMPILER_UDL
 #    endif
-#    ifdef _MSC_VER
-#      if _MSC_VER == 1700
-//       <initializer_list> is missing with MSVC 2012 (it's present in 2010, 2013 and up)
-#        undef Q_COMPILER_INITIALIZER_LISTS
-#      endif
-#      if _MSC_VER < 1900
-//       ICC disables unicode string support when compatibility mode with MSVC 2013 or lower is active
-#        undef Q_COMPILER_UNICODE_STRINGS
-//       Even though ICC knows about ref-qualified members, MSVC 2013 or lower doesn't, so
-//       certain member functions (like QString::toUpper) may be missing from the DLLs.
-#        undef Q_COMPILER_REF_QUALIFIERS
-//       Disable constexpr unless the MS headers have constexpr in all the right places too
-//       (like std::numeric_limits<T>::max())
-#        undef Q_COMPILER_CONSTEXPR
-#      endif
+#  elif defined(__STDC_VERSION__) && __STDC_VERSION__ > 199901L
+//   C11 features supported. Only tested with ICC 17 and up.
+#    define Q_COMPILER_STATIC_ASSERT
+#    if __has_include(<threads.h>)
+#      define Q_COMPILER_THREAD_LOCAL
 #    endif
 #  endif
 #endif
@@ -756,7 +778,7 @@
 #      define Q_COMPILER_TEMPLATE_ALIAS
 #    endif
 #    if __has_feature(cxx_thread_local)
-#      if !defined(__FreeBSD__) && !defined(__OpenBSD__) /* FreeBSD clang fails on __cxa_thread_atexit */
+#      if !defined(__FreeBSD__) /* FreeBSD clang fails on __cxa_thread_atexit */
 #        define Q_COMPILER_THREAD_LOCAL
 #      endif
 #    endif
@@ -803,15 +825,27 @@
 #    endif
 #  endif
 
-#  if defined(__has_warning)
-#    if __has_warning("-Wunused-private-field")
-#      define Q_DECL_UNUSED_MEMBER Q_DECL_UNUSED
+#  if defined(__STDC_VERSION__)
+#    if __has_feature(c_static_assert)
+#      define Q_COMPILER_STATIC_ASSERT
+#    endif
+#    if __has_feature(c_thread_local) && __has_include(<threads.h>)
+#      if !defined(__FreeBSD__) /* FreeBSD clang fails on __cxa_thread_atexit */
+#        define Q_COMPILER_THREAD_LOCAL
+#      endif
 #    endif
 #  endif
 
-#endif // Q_CC_CLANG
+#endif // Q_CC_CLANG &&  !Q_CC_INTEL && !Q_CC_MSVC
 
-#if defined(Q_CC_GNU) && !defined(Q_CC_INTEL) && !defined(Q_CC_CLANG)
+#if defined(Q_CC_CLANG) && !defined(Q_CC_INTEL)
+#  ifndef Q_DECL_UNUSED
+#    define Q_DECL_UNUSED __attribute__((__unused__))
+#  endif
+#  define Q_DECL_UNUSED_MEMBER Q_DECL_UNUSED
+#endif
+
+#if defined(Q_CC_GNU_ONLY)
 #  define Q_COMPILER_RESTRICTED_VLA
 #  define Q_COMPILER_THREADSAFE_STATICS
 #  if Q_CC_GNU >= 403
@@ -857,7 +891,6 @@
 #      define Q_COMPILER_DEFAULT_MEMBERS
 #      define Q_COMPILER_DELETE_MEMBERS
        /* C++11 features supported in GCC 4.6: */
-#      define Q_COMPILER_CONSTEXPR
 #      define Q_COMPILER_NULLPTR
 #      define Q_COMPILER_UNRESTRICTED_UNIONS
 #      define Q_COMPILER_RANGE_FOR
@@ -888,7 +921,11 @@
 #         define Q_COMPILER_REF_QUALIFIERS
 #      endif
 #    endif
-     /* C++11 features are complete as of GCC 4.8.1 */
+#    if Q_CC_GNU >= 500
+       /* GCC 4.6 introduces constexpr, but it's bugged (at least) in the whole
+        * 4.x series, see e.g. https://gcc.gnu.org/bugzilla/show_bug.cgi?id=57694 */
+#      define Q_COMPILER_CONSTEXPR
+#    endif
 #  endif
 #  if __cplusplus > 201103L
 #    if Q_CC_GNU >= 409
@@ -898,15 +935,25 @@
 #      define Q_COMPILER_RETURN_TYPE_DEDUCTION
 #    endif
 #  endif
+#  if defined(__STDC_VERSION__) && __STDC_VERSION__ > 199901L
+#    if Q_CC_GNU >= 407
+       /* C11 features supported in GCC 4.7: */
+#      define Q_COMPILER_STATIC_ASSERT
+#    endif
+#    if Q_CC_GNU >= 409 && defined(__has_include)
+       /* C11 features supported in GCC 4.9: */
+#      if __has_include(<threads.h>)
+#        define Q_COMPILER_THREAD_LOCAL
+#      endif
+#    endif
+#  endif
 #endif
 
 #if defined(Q_CC_MSVC)
 #  if defined(__cplusplus)
-#    if _MSC_VER >= 1400
        /* C++11 features supported in VC8 = VC2005: */
 #      define Q_COMPILER_VARIADIC_MACROS
 
-#      ifndef __cplusplus_cli
        /* 2005 supports the override and final contextual keywords, in
         the same positions as the C++11 variants, but 'final' is
         called 'sealed' instead:
@@ -915,12 +962,7 @@
         "virtual" keyword to be present too, so don't define for that.
         So don't define Q_COMPILER_EXPLICIT_OVERRIDES (since it's not
         the same as the C++11 version), but define the Q_DECL_* flags
-        accordingly: */
-#      define Q_DECL_OVERRIDE override
-#      define Q_DECL_FINAL sealed
-#      endif
-#    endif
-#    if _MSC_VER >= 1600
+        accordingly. */
        /* C++11 features supported in VC10 = VC2010: */
 #      define Q_COMPILER_AUTO_FUNCTION
 #      define Q_COMPILER_AUTO_TYPE
@@ -930,57 +972,31 @@
 #      define Q_COMPILER_NULLPTR
 #      define Q_COMPILER_RVALUE_REFS
 #      define Q_COMPILER_STATIC_ASSERT
-//  MSVC's library has std::initializer_list, but the compiler does not support the braces initialization
-//#      define Q_COMPILER_INITIALIZER_LISTS
-//#      define Q_COMPILER_UNIFORM_INIT
-#    endif
-#    if _MSC_VER >= 1700
        /* C++11 features supported in VC11 = VC2012: */
-#       undef Q_DECL_OVERRIDE               /* undo 2005/2008 settings... */
-#       undef Q_DECL_FINAL                  /* undo 2005/2008 settings... */
 #      define Q_COMPILER_EXPLICIT_OVERRIDES /* ...and use std C++11 now   */
 #      define Q_COMPILER_CLASS_ENUM
 #      define Q_COMPILER_ATOMICS
-#    endif /* VC 11 */
-#    if _MSC_VER >= 1800
        /* C++11 features in VC12 = VC2013 */
-/* Implemented, but can't be used on move special members */
-/* #      define Q_COMPILER_DEFAULT_MEMBERS */
 #      define Q_COMPILER_DELETE_MEMBERS
 #      define Q_COMPILER_DELEGATING_CONSTRUCTORS
 #      define Q_COMPILER_EXPLICIT_CONVERSIONS
 #      define Q_COMPILER_NONSTATIC_MEMBER_INIT
-// implemented, but nested initialization fails (eg tst_qvector): http://connect.microsoft.com/VisualStudio/feedback/details/800364/initializer-list-calls-object-destructor-twice
-//      #define Q_COMPILER_INITIALIZER_LISTS
-// implemented in principle, but has a bug that makes it unusable: http://connect.microsoft.com/VisualStudio/feedback/details/802058/c-11-unified-initialization-fails-with-c-style-arrays
-//      #define Q_COMPILER_UNIFORM_INIT
 #      define Q_COMPILER_RAW_STRINGS
 #      define Q_COMPILER_TEMPLATE_ALIAS
 #      define Q_COMPILER_VARIADIC_TEMPLATES
-#    endif /* VC 12 */
-#    if _MSC_FULL_VER >= 180030324 // VC 12 SP 2 RC
-#      define Q_COMPILER_INITIALIZER_LISTS
-#    endif /* VC 12 SP 2 RC */
-#    if _MSC_VER >= 1900
+#      define Q_COMPILER_INITIALIZER_LISTS // VC 12 SP 2 RC
        /* C++11 features in VC14 = VC2015 */
 #      define Q_COMPILER_DEFAULT_MEMBERS
 #      define Q_COMPILER_ALIGNAS
 #      define Q_COMPILER_ALIGNOF
-// Partial support, insufficient for Qt
-//#      define Q_COMPILER_CONSTEXPR
 #      define Q_COMPILER_INHERITING_CONSTRUCTORS
 #      define Q_COMPILER_NOEXCEPT
 #      define Q_COMPILER_RANGE_FOR
 #      define Q_COMPILER_REF_QUALIFIERS
 #      define Q_COMPILER_THREAD_LOCAL
-// Broken, see QTBUG-47224 and https://connect.microsoft.com/VisualStudio/feedback/details/1549785
-//#      define Q_COMPILER_THREADSAFE_STATICS
 #      define Q_COMPILER_UDL
 #      define Q_COMPILER_UNICODE_STRINGS
-// Uniform initialization is not working yet -- build errors with QUuid
-//#      define Q_COMPILER_UNIFORM_INIT
 #      define Q_COMPILER_UNRESTRICTED_UNIONS
-#    endif
 #    if _MSC_FULL_VER >= 190023419
 #      define Q_COMPILER_ATTRIBUTES
 // Almost working, see https://connect.microsoft.com/VisualStudio/feedback/details/2011648
@@ -1060,37 +1076,6 @@
 # endif
 #endif
 
-/*
- * SG10's SD-6 feature detection and some useful extensions from Clang and GCC
- * https://isocpp.org/std/standing-documents/sd-6-sg10-feature-test-recommendations
- * http://clang.llvm.org/docs/LanguageExtensions.html#feature-checking-macros
- */
-#ifdef __has_builtin
-#  define QT_HAS_BUILTIN(x)             __has_builtin(x)
-#else
-#  define QT_HAS_BUILTIN(x)             0
-#endif
-#ifdef __has_attribute
-#  define QT_HAS_ATTRIBUTE(x)           __has_attribute(x)
-#else
-#  define QT_HAS_ATTRIBUTE(x)           0
-#endif
-#ifdef __has_cpp_attribute
-#  define QT_HAS_CPP_ATTRIBUTE(x)       __has_cpp_attribute(x)
-#else
-#  define QT_HAS_CPP_ATTRIBUTE(x)       0
-#endif
-#ifdef __has_include
-#  define QT_HAS_INCLUDE(x)             __has_include(x)
-#else
-#  define QT_HAS_INCLUDE(x)             0
-#endif
-#ifdef __has_include_next
-#  define QT_HAS_INCLUDE_NEXT(x)        __has_include_next(x)
-#else
-#  define QT_HAS_INCLUDE_NEXT(x)        0
-#endif
-
 /*
  * C++11 keywords and expressions
  */
@@ -1151,16 +1136,11 @@
 #ifdef Q_COMPILER_NOEXCEPT
 # define Q_DECL_NOEXCEPT noexcept
 # define Q_DECL_NOEXCEPT_EXPR(x) noexcept(x)
-# ifdef Q_DECL_NOTHROW
-#  undef Q_DECL_NOTHROW /* override with C++11 noexcept if available */
-# endif
 #else
 # define Q_DECL_NOEXCEPT
 # define Q_DECL_NOEXCEPT_EXPR(x)
 #endif
-#ifndef Q_DECL_NOTHROW
-# define Q_DECL_NOTHROW Q_DECL_NOEXCEPT
-#endif
+#define Q_DECL_NOTHROW Q_DECL_NOEXCEPT
 
 #if defined(Q_COMPILER_ALIGNOF)
 #  undef Q_ALIGNOF
@@ -1172,12 +1152,25 @@
 #  define Q_DECL_ALIGN(n)   alignas(n)
 #endif
 
-#if QT_HAS_CPP_ATTRIBUTE(nodiscard) && !defined(Q_CC_CLANG)         // P0188R1
+#if __has_cpp_attribute(nodiscard) && !defined(Q_CC_CLANG)         // P0188R1
 // Can't use [[nodiscard]] with Clang, see https://bugs.llvm.org/show_bug.cgi?id=33518
 #  undef Q_REQUIRED_RESULT
 #  define Q_REQUIRED_RESULT [[nodiscard]]
 #endif
 
+#if defined(__cpp_enumerator_attributes) && __cpp_enumerator_attributes >= 201411
+#if defined(Q_CC_MSVC)
+// Can't mark enum values as __declspec(deprecated) with MSVC, also can't move
+// everything to [[deprecated]] because MSVC gives a compilation error when marking
+// friend methods of a class as [[deprecated("text")]], breaking qstring.h
+#  define Q_DECL_ENUMERATOR_DEPRECATED [[deprecated]]
+#  define Q_DECL_ENUMERATOR_DEPRECATED_X(x) [[deprecated(x)]]
+#else
+#  define Q_DECL_ENUMERATOR_DEPRECATED Q_DECL_DEPRECATED
+#  define Q_DECL_ENUMERATOR_DEPRECATED_X(x) Q_DECL_DEPRECATED_X(x)
+#endif
+#endif
+
 /*
  * Fallback macros to certain compiler features
  */
@@ -1212,6 +1205,12 @@
 #ifndef Q_DECL_DEPRECATED_X
 #  define Q_DECL_DEPRECATED_X(text) Q_DECL_DEPRECATED
 #endif
+#ifndef Q_DECL_ENUMERATOR_DEPRECATED
+#  define Q_DECL_ENUMERATOR_DEPRECATED
+#endif
+#ifndef Q_DECL_ENUMERATOR_DEPRECATED_X
+#  define Q_DECL_ENUMERATOR_DEPRECATED_X(x)
+#endif
 #ifndef Q_DECL_EXPORT
 #  define Q_DECL_EXPORT
 #endif
@@ -1246,17 +1245,21 @@
 #ifndef Q_DECL_CONST_FUNCTION
 #  define Q_DECL_CONST_FUNCTION Q_DECL_PURE_FUNCTION
 #endif
+#ifndef Q_DECL_COLD_FUNCTION
+#  define Q_DECL_COLD_FUNCTION
+#endif
 #ifndef QT_MAKE_UNCHECKED_ARRAY_ITERATOR
 #  define QT_MAKE_UNCHECKED_ARRAY_ITERATOR(x) (x)
 #endif
 #ifndef QT_MAKE_CHECKED_ARRAY_ITERATOR
 #  define QT_MAKE_CHECKED_ARRAY_ITERATOR(x, N) (x)
 #endif
-#ifdef __has_feature
-#  define QT_HAS_FEATURE(x)             __has_feature(x)
-#else
-#  define QT_HAS_FEATURE(x)             0
-#endif
+
+/*
+ * "Weak overloads" - makes an otherwise confliciting overload weaker
+ * (by making it a template)
+ */
+#define Q_WEAK_OVERLOAD template <typename = void>
 
 /*
  * Warning/diagnostic handling
@@ -1282,7 +1285,7 @@
 #  define QT_WARNING_DISABLE_CLANG(text)
 #  define QT_WARNING_DISABLE_GCC(text)
 #  define QT_WARNING_DISABLE_DEPRECATED         QT_WARNING_DISABLE_INTEL(1478 1786)
-#elif defined(Q_CC_MSVC) && _MSC_VER >= 1500 && !defined(Q_CC_CLANG)
+#elif defined(Q_CC_MSVC) && !defined(Q_CC_CLANG)
 #  undef QT_DO_PRAGMA                           /* not needed */
 #  define QT_WARNING_PUSH                       __pragma(warning(push))
 #  define QT_WARNING_POP                        __pragma(warning(pop))
@@ -1318,6 +1321,14 @@
 #  define QT_WARNING_DISABLE_DEPRECATED
 #endif
 
+#ifndef QT_IGNORE_DEPRECATIONS
+#define QT_IGNORE_DEPRECATIONS(statement) \
+    QT_WARNING_PUSH \
+    QT_WARNING_DISABLE_DEPRECATED \
+    statement \
+    QT_WARNING_POP
+#endif
+
 /*
    Proper for-scoping in MIPSpro CC
 */
@@ -1347,11 +1358,11 @@
     } while (false)
 
 #if defined(__cplusplus)
-#if QT_HAS_CPP_ATTRIBUTE(clang::fallthrough)
+#if __has_cpp_attribute(clang::fallthrough)
 #    define Q_FALLTHROUGH() [[clang::fallthrough]]
-#elif QT_HAS_CPP_ATTRIBUTE(gnu::fallthrough)
+#elif __has_cpp_attribute(gnu::fallthrough)
 #    define Q_FALLTHROUGH() [[gnu::fallthrough]]
-#elif QT_HAS_CPP_ATTRIBUTE(fallthrough)
+#elif __has_cpp_attribute(fallthrough)
 #  define Q_FALLTHROUGH() [[fallthrough]]
 #endif
 #endif
@@ -1375,6 +1386,7 @@
 #  undef QT_COMPILER_SUPPORTS_SSE4_2
 #  undef QT_COMPILER_SUPPORTS_AVX
 #  undef QT_COMPILER_SUPPORTS_AVX2
+#  undef QT_COMPILER_SUPPORTS_F16C
 #endif
 #if !defined(Q_PROCESSOR_ARM)
 #  undef QT_COMPILER_SUPPORTS_NEON
